function [xk,zk,uk,yk,feas]=LPFI(c,A,b,x0,z0,u0,y0,tol,Q)
%
%   
% S. Ulbrich, June 2002
%
% This code comes with no guarantee or warranty of any kind.
%
% function [xk,zk,uk,yk]=LPFI(c,A,b,x0,z0,u0,y0,tol,Q)
% 
% Infeasible long step path following algorithm LPF
% solves the LP or convex QP
%
%             [x]'  [x]      [x]         [x]
%    min  0.5*[z]*Q*[z]+ c'* [z]  s.t. A [z] =b,   x>=0
%
%                               [x]
% The optimality system with w= [z]  is given by
%       A w=b,   x>=0
%                  [y]
%       Q*w+c-A'*u-[0]=0, y>=0
%       X y =0
%
%  Input:      Q,c,A,b:  Problem data: c in R^(nf+n),
%                                      Q in R^(nf+n,nf+n) psd (optional),
%                                      A in R^(m,nf+n), b in R^m
%          x0,z0,u0,y0:  Initial data
%                        x0 in R^n, z0 in R^nf, u0 in R^m, y0 in R^n
%                        x0,y0>0
%                  tol:  Stopping tolerance
%                        The algorithm stops if xk'*yk/n<=tol
%
%  Output: xk,zk,uk,yk: Primal solution [xk,zk] and dual solution [uk,yk]
%
%
n=size(x0,1);
nf=size(z0,1);
m=size(b,1);
% default constants for central path neighborhood N-inf(ga,be)
ga0=0.01;
be=2;
% centering parameters for fast and safe step
sigmin=0.01;
sigmax=0.3;
if nargin<9  % LP, since Q=[]
 Q=sparse(n+nf,n+nf);
end
xk=x0;
yk=y0;
zk=z0;
uk=u0;
muk=xk'*yk/n;
mu0=muk;
% adjust ga such that [z0,x0,u0,y0] in N-inf(ga,be)
ga=min(ga0,min(xk.*yk)/muk);
if nargin<9
 rD=c-A'*uk-[yk;0*zk];
else
 rD=Q*[xk;zk]+c-A'*uk-[yk;0*zk];
end
rP=A*[xk;zk]-b;
resk=norm([rP;rD]);
res0=resk;
it=1;
alk=1;
while (muk>tol) & (alk>1e-5)
 Xk=spdiags(xk,0,n,n);
 Yk=spdiags(yk,0,n,n);
 DF=[[Q -A' [-speye(n);sparse(nf,n)]];...
     [A sparse(m,m+n)];[Yk sparse(n,m+nf) Xk]];
% compute column minimum degree ordering to improve sparsity of
% LU-factors
 if (it==1)
  P=colmmd(DF);
 end
% try a fast step with small centering parameter sigk=sigmin
 sigk=sigmin;
 F=[rD;rP;Xk*yk-sigk*muk];
 [L,U]=lu(DF(:,P));
 dwk=-L\F;
 dwk=U\dwk;
 dwk(P)=dwk;
 if (nf>0)
  dzk=dwk(n+1:n+nf);
 end
 dxk=dwk(1:n);
 duk=dwk(n+nf+1:n+nf+m);
 dyk=dwk(n+nf+m+1:2*n+nf+m);
% compute maximal stepsize with [zk,xk,uk,yk] in N-inf(ga,be)
 alk=calcalNinf(ga,xk,yk,dxk,dyk);
 alk1=calcalmures(alk,be,xk,yk,dxk,dyk,mu0,res0,resk);
 alk=min(alk1,alk);
% if alk>=0 take fast step else take safe step with sigk=sigmax
 if (alk<0.5)
  sigk=0.3;
  F=[rD;rP;Xk*yk-sigk*muk];
  dwk=-L\F;
  dwk=U\dwk;
  dwk(P)=dwk;
  if (nf>0)
   dzk=dwk(n+1:n+nf);
  end
  dxk=dwk(1:n);
  duk=dwk(n+nf+1:n+nf+m);
  dyk=dwk(n+nf+m+1:2*n+nf+m);
  alk=calcalNinf(ga,xk,yk,dxk,dyk);
  alk1=calcalmures(alk,be,xk,yk,dxk,dyk,mu0,res0,resk);
  alk=min(alk1,alk);
 end
 xk=xk+alk*dxk;
 yk=yk+alk*dyk;
 if (nf>0)
  zk=zk+alk*dzk;
 end
 uk=uk+alk*duk;
 muk=xk'*yk/n;
 if nargin<9
  rD=c-A'*uk-[yk;0*zk];
 else
  rD=Q*[xk;zk]+c-A'*uk-[yk;0*zk];
 end
 rP=A*[xk;zk]-b;
 resk=norm([rP;rD]);
 fprintf(1,'it=%3.d   muk=%e   resk=%e  alk=%e\n',it,muk,resk,alk);
 it=it+1;
end
feas=1;
if (alk<=1e-5)
   feas=0;
end
